#! /usr/bin/python
#
# Use this script as thumbnailer with Sketchup in Wine @ Linux:
# 1. Save the script in /usr/bin
#    (for example open the terminal and type: gksudo nautilus /usr/bin)
# 2. Register this thumbnailer in the Gnome configuration:
#    For Gnome 3, you need a small registration file in /usr/share/thumbnailers.
#    For Gnome 2, you create the following gconf keys using the terminal:
# gconftool-2 -s /desktop/gnome/thumbnailers/application@x-wine-extension-skp/enable --type=bool true
# gconftool-2 -s /desktop/gnome/thumbnailers/application@x-wine-extension-skp/command --type=string "python /usr/bin/skp-thumbnailer.py %i %u %o %s"
#
# Usually thumbnailers are registered in Gnome and then envoked
# when needed. 
# [Gnome 2] http://developer.gnome.org/integration-guide/stable/thumbnailer.html.en
# 
# SketchUp files contain an embedded 128px thumbnail that this script is able to extract on demand.
#
# However large thumbnails can only be created from within SketchUp, and launching
# SketchUp takes some time.
# So we generate thumbnails for every file that is opened in SketchUp
# and store the thumbnail in the thumbnail cache.
# According to the Free Desktop Specification the thumbnail cache is in 
# $XDG_CACHE_HOME/thumbnails/large/<md5 hash of target file URI>.png
# (~/.thumbnails or ~/.cache/thumbnails)
# http://specifications.freedesktop.org/thumbnail-spec/thumbnail-spec-latest.html#THUMBSAVE
# 
# As it seems, Ubuntu Unity does not directly access large thumbnails, but
# keeps only a session cache in ~/.local/share/previews using a shorter 
# C++ std::hash<std::string> hash_fn. 
# http://bazaar.launchpad.net/~unity-team/unity/trunk/view/head:/dash/previews/GenericPreview.cpp
# Since we can not create skp
# previews on demand without launching SketchUp and since we cannot
# predict or reproduce the C++ hash, we use this script to copy the image file 
# from the Free Desktop thumbnail cache to the Unity preview cache.

import sys
import struct
import os
import hashlib
import urllib

target_file = sys.argv[1] if sys.argv[1].startswith("/") else urllib.unquote(sys.argv[4].replace("file://","",1))
thumb_file = sys.argv[2]
size = sys.argv[3] if len(sys.argv)>=3 and not sys.argv[3].startswith("%") else 128

counter=0
def pngcopy(infile, outfile):
# copy header
    header = infile.read(8)
#    if header != "\211PNG\r\n\032\n":
#        raise IOError("not a valid PNG file")
    outfile.write(header)
    print header
# copy chunks, until IEND
    global counter
    while 1:
        chunk = infile.read(8)
        size, cid = struct.unpack("!l4s", chunk)
        print counter,chunk
        outfile.write(chunk)
        outfile.write(infile.read(size))
        outfile.write(infile.read(4)) # checksum
        counter+=1
        if cid == "IEND":
            break


# If a large thumbnail is requested, check if one is in the cache.
if size>128:
    # Get expected path of large thumbnail (thumbnail cache/large/<md5 of target_file URI>.png).
    input_uri = "file://"+urllib.quote(target_file)
    print input_uri
    h = hashlib.md5(input_uri).hexdigest()
    print h
    cache_dir = "%s/thumbnails" % os.environ["XDG_CACHE_HOME"] if "XDG_CACHE_HOME" in os.environ else "%s/.cache/thumbnails" % os.environ["HOME"]
    if not os.path.exists(cache_dir):
        cache_dir = "%s/.thumbnails" % os.environ["HOME"]
    cached_file = os.path.join(cache_dir, "large", h+".png")
    print "file exists "+`cached_file`
    if os.path.exists(cached_file) and not os.path.exists(thumb_file):
        # Link to the existing file.
        os.symlink(cached_file, thumb_file)
        exit(0)
    else:
        print "file not found "+`cached_file`

# For a small thumbnail, extract it out of the skp.
infile = open(target_file, "rb")
infile.seek(0x76d)
outfile = open(thumb_file, "wb")
pngcopy(infile, outfile)
outfile.close()
# Remove the thumbnail if it is empty (ie. because of not working png exporter in SketchUp in Wine)
if counter<=5:
    os.remove(thumb_file)
infile.close()
exit(0)
